@@ -17,11 +17,9 @@ import (
 	"context"
 	"errors"
 
-	"github.com/prometheus/client_golang/api"
-	"gopkg.in/alecthomas/kingpin.v2"
+	kingpin "github.com/alecthomas/kingpin/v2"
 
 	"github.com/prometheus/alertmanager/cli/format"
-	"github.com/prometheus/alertmanager/client"
 )
 
 const configHelp = `View current config.
@@ -32,19 +30,15 @@ The amount of output is controlled by the output selection flag:
 	- Json: Print entire config object as json
 `
 
-// configCmd represents the config command
+// configureConfigCmd represents the config command.
 func configureConfigCmd(app *kingpin.Application) {
-	app.Command("config", configHelp).Action(queryConfig).PreAction(requireAlertManagerURL)
-
+	configCmd := app.Command("config", configHelp)
+	configCmd.Command("show", configHelp).Default().Action(execWithTimeout(queryConfig)).PreAction(requireAlertManagerURL)
+	configureRoutingCmd(configCmd)
 }
 
-func queryConfig(ctx *kingpin.ParseContext) error {
-	c, err := api.NewClient(api.Config{Address: alertmanagerURL.String()})
-	if err != nil {
-		return err
-	}
-	statusAPI := client.NewStatusAPI(c)
-	status, err := statusAPI.Get(context.Background())
+func queryConfig(ctx context.Context, _ *kingpin.ParseContext) error {
+	status, err := getRemoteAlertmanagerConfigStatus(ctx, alertmanagerURL)
 	if err != nil {
 		return err
 	}
