@@ -18,19 +18,22 @@ import (
 	"net/url"
 	"os"
 	"path"
+	"strings"
 	"time"
 
 	"github.com/alecthomas/kingpin/v2"
+	clientruntime "github.com/go-openapi/runtime/client"
 	"github.com/go-openapi/strfmt"
 	promconfig "github.com/prometheus/common/config"
+	"github.com/prometheus/common/promslog"
 	"github.com/prometheus/common/version"
 	"golang.org/x/mod/semver"
 
 	"github.com/prometheus/alertmanager/api/v2/client"
 	"github.com/prometheus/alertmanager/cli/config"
 	"github.com/prometheus/alertmanager/cli/format"
-
-	clientruntime "github.com/go-openapi/runtime/client"
+	"github.com/prometheus/alertmanager/featurecontrol"
+	"github.com/prometheus/alertmanager/matcher/compat"
 )
 
 var (
@@ -40,11 +43,27 @@ var (
 	timeout         time.Duration
 	httpConfigFile  string
 	versionCheck    bool
+	featureFlags    string
 
 	configFiles = []string{os.ExpandEnv("$HOME/.config/amtool/config.yml"), "/etc/amtool/config.yml"}
 	legacyFlags = map[string]string{"comment_required": "require-comment"}
 )
 
+func initMatchersCompat(_ *kingpin.ParseContext) error {
+	promslogConfig := &promslog.Config{Writer: os.Stdout}
+	if verbose {
+		promslogConfig.Level = &promslog.AllowedLevel{}
+		_ = promslogConfig.Level.Set("debug")
+	}
+	logger := promslog.New(promslogConfig)
+	featureConfig, err := featurecontrol.NewFlags(logger, featureFlags)
+	if err != nil {
+		kingpin.Fatalf("error parsing the feature flag list: %v\n", err)
+	}
+	compat.InitFromFlags(logger, featureConfig)
+	return nil
+}
+
 func requireAlertManagerURL(pc *kingpin.ParseContext) error {
 	// Return without error if any help flag is set.
 	for _, elem := range pc.Elements {
@@ -69,7 +88,7 @@ const (
 	defaultAmApiv2path = "/api/v2"
 )
 
-// NewAlertmanagerClient initializes an alertmanager client with the given URL
+// NewAlertmanagerClient initializes an alertmanager client with the given URL.
 func NewAlertmanagerClient(amURL *url.URL) *client.AlertmanagerAPI {
 	address := defaultAmHost + ":" + defaultAmPort
 	schemes := []string{"http"}
@@ -125,7 +144,7 @@ func NewAlertmanagerClient(amURL *url.URL) *client.AlertmanagerAPI {
 	return c
 }
 
-// Execute is the main function for the amtool command
+// Execute is the main function for the amtool command.
 func Execute() {
 	app := kingpin.New("amtool", helpRoot).UsageWriter(os.Stdout)
 
@@ -137,6 +156,7 @@ func Execute() {
 	app.Flag("timeout", "Timeout for the executed command").Default("30s").DurationVar(&timeout)
 	app.Flag("http.config.file", "HTTP client configuration file for amtool to connect to Alertmanager.").PlaceHolder("<filename>").ExistingFileVar(&httpConfigFile)
 	app.Flag("version-check", "Check alertmanager version. Use --no-version-check to disable.").Default("true").BoolVar(&versionCheck)
+	app.Flag("enable-feature", fmt.Sprintf("Experimental features to enable, comma separated. Valid options: %s", strings.Join(featurecontrol.AllowedFlags, ", "))).Default("").StringVar(&featureFlags)
 
 	app.Version(version.Print("amtool"))
 	app.GetFlag("help").Short('h')
@@ -154,6 +174,8 @@ func Execute() {
 	configureConfigCmd(app)
 	configureTemplateCmd(app)
 
+	app.Action(initMatchersCompat)
+
 	err = resolver.Bind(app, os.Args[1:])
 	if err != nil {
 		kingpin.Fatalf("%v\n", err)
