@@ -15,11 +15,10 @@ package inhibit
 
 import (
 	"context"
+	"log/slog"
 	"sync"
 	"time"
 
-	"github.com/go-kit/log"
-	"github.com/go-kit/log/level"
 	"github.com/oklog/run"
 	"github.com/prometheus/common/model"
 
@@ -36,15 +35,15 @@ import (
 type Inhibitor struct {
 	alerts provider.Alerts
 	rules  []*InhibitRule
-	marker types.Marker
-	logger log.Logger
+	marker types.AlertMarker
+	logger *slog.Logger
 
 	mtx    sync.RWMutex
 	cancel func()
 }
 
 // NewInhibitor returns a new Inhibitor.
-func NewInhibitor(ap provider.Alerts, rs []config.InhibitRule, mk types.Marker, logger log.Logger) *Inhibitor {
+func NewInhibitor(ap provider.Alerts, rs []config.InhibitRule, mk types.AlertMarker, logger *slog.Logger) *Inhibitor {
 	ih := &Inhibitor{
 		alerts: ap,
 		marker: mk,
@@ -67,14 +66,14 @@ func (ih *Inhibitor) run(ctx context.Context) {
 			return
 		case a := <-it.Next():
 			if err := it.Err(); err != nil {
-				level.Error(ih.logger).Log("msg", "Error iterating alerts", "err", err)
+				ih.logger.Error("Error iterating alerts", "err", err)
 				continue
 			}
 			// Update the inhibition rules' cache.
 			for _, r := range ih.rules {
 				if r.SourceMatchers.Matches(a.Labels) {
 					if err := r.scache.Set(a); err != nil {
-						level.Error(ih.logger).Log("msg", "error on set alert", "err", err)
+						ih.logger.Error("error on set alert", "err", err)
 					}
 				}
 			}
@@ -106,7 +105,7 @@ func (ih *Inhibitor) Run() {
 	})
 
 	if err := g.Run(); err != nil {
-		level.Warn(ih.logger).Log("msg", "error running inhibitor", "err", err)
+		ih.logger.Warn("error running inhibitor", "err", err)
 	}
 }
 
@@ -215,7 +214,7 @@ func NewInhibitRule(cr config.InhibitRule) *InhibitRule {
 
 	equal := map[model.LabelName]struct{}{}
 	for _, ln := range cr.Equal {
-		equal[ln] = struct{}{}
+		equal[model.LabelName(ln)] = struct{}{}
 	}
 
 	return &InhibitRule{
@@ -231,10 +230,11 @@ func NewInhibitRule(cr config.InhibitRule) *InhibitRule {
 // is returned. If excludeTwoSidedMatch is true, alerts that match both the
 // source and the target side of the rule are disregarded.
 func (r *InhibitRule) hasEqual(lset model.LabelSet, excludeTwoSidedMatch bool) (model.Fingerprint, bool) {
+	now := time.Now()
 Outer:
 	for _, a := range r.scache.List() {
 		// The cache might be stale and contain resolved alerts.
-		if a.Resolved() {
+		if a.ResolvedAt(now) {
 			continue
 		}
 		for n := range r.Equal {
